using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace FastGraphics
{
    public class Program
    {
        public static void Main( string[] args )
        {
#if DEBUG
            System.Diagnostics.Debug.Listeners.Add( new System.Diagnostics.TextWriterTraceListener( Console.Out ));
#endif
            Application.Run( new FastGraphics() );
        }
    }

    internal class FastGraphics : Form
    {
        private DrawBox m_DrawBox;
        private LayerManager m_LayerManager;

        public FastGraphics()
        {
            // Creating the Form
            this.FormBorderStyle = FormBorderStyle.FixedSingle;
            this.MinimizeBox = false;
            this.MaximizeBox = false;
            this.Text = "Fast Graphics Demo";
            this.ClientSize = new Size( 800, 400 );
            m_DrawBox = new DrawBox( this.ClientSize.Width, this.ClientSize.Height );
            m_DrawBox.Location = new Point( 0, 0 );

            this.Controls.Add( m_DrawBox );
            
            // Initializing the Layer Manager and the Layers    
            m_LayerManager = new LayerManager( m_DrawBox );

            Layer layerBackground = new Layer( "Hintergrund", AppResources.parallax_hintergrund, Layer.LayerMovement.None, 0 );
            m_LayerManager.Add( layerBackground );
            Layer layerMountains = new Layer( "Berge", AppResources.parallax_berge, Layer.LayerMovement.Left, 3 );
            m_LayerManager.Add( layerMountains );
            Layer layerGrass = new Layer( "Gras", AppResources.parallax_gras, Layer.LayerMovement.Left, 6 );
            m_LayerManager.Add( layerGrass );
            Layer layerTrees = new Layer( "Bume", AppResources.parallax_baeume, Layer.LayerMovement.Left, 9 );
            m_LayerManager.Add( layerTrees );

            // Periodically updating the Drawing Area
            Timer repaintTimer = new Timer();
            repaintTimer.Interval = 20;         // Repaint Timer is triggered every 20ms
            repaintTimer.Tick += delegate( object sender, EventArgs e )
            {
                m_LayerManager.Update();
            };
            repaintTimer.Start();

            // Write FPS to title area
            Timer fpsTimer = new Timer();
            fpsTimer.Interval = 1000;
            fpsTimer.Tick += delegate( object sender, EventArgs e )
            {
                this.Text = String.Format( "FPS: {0}", m_LayerManager.FramesDrawn );
                m_LayerManager.FramesDrawn = 0;
            };
            fpsTimer.Start();
        }
    }
}