using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;
using System.Diagnostics;

namespace FastGraphics
{
    internal class Layer
    {
        private string m_LayerName;
        private Image m_LayerImage;
        private Bitmap m_ViewBuffer;
        private LayerMovement m_Movement;
        private int m_Speed;
        private int m_CurrentOffset;
        private Rectangle m_SourceRect;
        private Rectangle m_DestRect;

        public Layer( string name, Image image, LayerMovement movement, int speed )
        {
            Debug.WriteLine( String.Format( "[DEBUG] Creating new Layer with the following Data:" ));
            Debug.WriteLine( String.Format( "[DEBUG]      Name = {0}", name ));
            Debug.WriteLine( String.Format( "[DEBUG]      Movement = {0}", movement.ToString() ));
            Debug.WriteLine( String.Format( "[DEBUG]      Speed = {0}", speed ));

            m_LayerName = name;
            m_LayerImage = image;
            m_Movement = movement;

            switch( movement )
            {
                case LayerMovement.Left:
                    m_Speed = speed;
                    break;

                case LayerMovement.Right:
                    m_Speed = speed * -1;
                    break;

                case LayerMovement.None:
                    m_Speed = 0;
                    break;
            }
            m_CurrentOffset = 0;

            m_SourceRect = new Rectangle();
            m_DestRect = new Rectangle();

            m_ViewBuffer = new Bitmap( m_LayerImage.Width, m_LayerImage.Height, PixelFormat.Format32bppPArgb );
        }

        public Image NextView
        {
            get
            {
                Debug.WriteLine( String.Format( "[DEBUG] Returning new Layer Perspective for Layer: {0}", m_LayerName ) );

                if( m_Speed == 0 ) return m_LayerImage;

                //Bitmap buffer = new Bitmap( m_LayerImage.Width, m_LayerImage.Height, PixelFormat.Format32bppPArgb );
                Graphics bufferGraphics = Graphics.FromImage( m_ViewBuffer );
                bufferGraphics.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighSpeed;
                bufferGraphics.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.Low;
                bufferGraphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.None;

                // Clearing the Bitmap (yes, it's slightly faster than creating a new Bitmap)
                bufferGraphics.Clear( Color.Transparent );

                // Even if it looks scary, we do not create the Source and Destination Rectangles
                // every time because this costs some unnecessary time (and yes, this method is performance critical)
                m_DestRect.X = 0;
                m_DestRect.Y = 0;
                m_DestRect.Width = m_LayerImage.Size.Width - m_CurrentOffset;
                m_DestRect.Height = m_LayerImage.Size.Height;
                m_SourceRect.X = m_CurrentOffset;
                m_SourceRect.Y = 0;
                m_SourceRect.Width = m_LayerImage.Size.Width - m_CurrentOffset;
                m_SourceRect.Height = m_LayerImage.Size.Height;

                bufferGraphics.DrawImage( m_LayerImage, m_DestRect, m_SourceRect, GraphicsUnit.Pixel );

                m_DestRect.X = m_LayerImage.Size.Width - m_CurrentOffset;
                m_DestRect.Y = 0;
                m_DestRect.Width = m_CurrentOffset;
                m_DestRect.Height = m_LayerImage.Size.Height;
                m_SourceRect.X = 0;
                m_SourceRect.Y = 0;
                m_SourceRect.Width = m_CurrentOffset;
                m_SourceRect.Height = m_LayerImage.Size.Height;

                bufferGraphics.DrawImage( m_LayerImage, m_DestRect, m_SourceRect, GraphicsUnit.Pixel );
                bufferGraphics.Dispose();

                m_CurrentOffset += m_Speed;
                if( m_CurrentOffset >= m_LayerImage.Size.Width ) m_CurrentOffset = 0;
                else if( m_CurrentOffset < 0 ) m_CurrentOffset = m_LayerImage.Size.Width;

                return m_ViewBuffer;
            }
        }

        public Image LayerImage
        {
            get { return m_LayerImage; }
        }

        public enum LayerMovement
        {
            Right,
            Left,
            None
        }
    }
}
