using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using System.Diagnostics;

namespace FastGraphics
{
    internal class LayerManager
    {       
        private List<Layer> m_Layers;
        private DrawBox m_DrawBox;
        private Bitmap m_BackBuffer;
        private int m_FramesDrawn = 0;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="drawBox">DrawBox Control to draw to</param>
        public LayerManager( DrawBox drawBox )
        {
            Debug.WriteLine( "[DEBUG] Layer Manager is initializing" );
            m_DrawBox = drawBox;
            m_Layers = new List<Layer>();
            m_BackBuffer = new Bitmap( m_DrawBox.Size.Width, m_DrawBox.Size.Height, PixelFormat.Format32bppPArgb );
        }

        /// <summary>
        /// Updates the DrawBox with the new content
        /// </summary>
        public void Update()
        {
            Debug.WriteLine( "[DEBUG] Layer Manager is updating the Layers" );

            Graphics bufferGraphics = Graphics.FromImage( m_BackBuffer );
            for( int index = 0; index < m_Layers.Count; index ++ ) // Using a for Loop for retrieving the elements is slightly faster than using the Iterator
            {
                bufferGraphics.DrawImageUnscaled( m_Layers[index].NextView, new Point( 0, 0 ));
            }
            bufferGraphics.Dispose();

            m_FramesDrawn ++;
            m_DrawBox.SetImage( m_BackBuffer );
            m_DrawBox.Invalidate();
        }

        /// <summary>
        /// The number of frames that have been drawn since the last reset
        /// </summary>
        public int FramesDrawn
        {
            get { return m_FramesDrawn; }
            set { m_FramesDrawn = value; }
        }

        /// <summary>
        /// Adds a new Layer to the Manager
        /// </summary>
        /// <param name="layer">Layer-Object to add</param>
        public void Add( Layer layer )
        {
            m_Layers.Add( layer );
        }
    }
}
